package com.huimv.datacollection.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.common.exception.RRException;
import com.huimv.datacollection.dao.DeviceDataDao;
import com.huimv.datacollection.entity.DeviceDataEntity;
import com.huimv.datacollection.entity.DeviceEntity;
import com.huimv.datacollection.entity.IndoorEnvironmentEntity;
import com.huimv.datacollection.enums.CmdEnum;
import com.huimv.datacollection.feign.ManagementFeignService;
import com.huimv.datacollection.service.HandleDataPackageService;
import com.huimv.datacollection.service.IClientDataPacketService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * @author yinhao
 * @version 1.0
 * @date 2021/07/07 15:07:00
 **/
@Service
@Slf4j
public class ClientDataPacketServiceImpl implements IClientDataPacketService {

    @Autowired
    private DeviceDataDao deviceDataDao;

    @Autowired
    private ManagementFeignService managementFeignService;

    @Autowired
    private HandleDataPackageService handleDataPackageService;

    @Override
    public void handleClientSendPacket(String clientMessageTxt) {

        if (clientMessageTxt == null || clientMessageTxt.isEmpty()) {
            log.error("本地提交数据包为空.");
            return;
        }

        String matcher = "[+]";
        String[] clientMessageArray = clientMessageTxt.split(matcher);
        String cmd = clientMessageArray[2];
        log.info("本次执行命令>>" + cmd);

        DeviceDataEntity deviceDataEntity = new DeviceDataEntity();
        deviceDataEntity.setCmdHeader(clientMessageArray[0]);
        String jqId = clientMessageArray[1];
        deviceDataEntity.setJqId(jqId);
        deviceDataEntity.setCmdType(Integer.parseInt(clientMessageArray[2]));
        deviceDataEntity.setSequence(Integer.parseInt(clientMessageArray[3]));
        Double value = null;
        if (CmdEnum.TEMPERATURE.getTypeNum().equals(cmd) || CmdEnum.HUMIDITY.getTypeNum().equals(cmd)) {
            value = new BigDecimal(clientMessageArray[4]).divide(new BigDecimal("10")).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        } else if (CmdEnum.GAS.getTypeNum().equals(cmd)) {
            value = Double.parseDouble(clientMessageArray[4]);
        }
        if (value == null) {
            throw new RRException("传入数据包的数据字段为空,请检查!");
        }
        deviceDataEntity.setValue(value);
        deviceDataEntity.setTime(clientMessageArray[5]);
        deviceDataEntity.setVerify(Integer.parseInt(clientMessageArray[6]));
        deviceDataEntity.setCmdEnd(clientMessageArray[7]);
        deviceDataDao.insert(deviceDataEntity);

        //获取设备信息并校验
        String deviceJson = JSONObject.toJSONString(managementFeignService.deviceInfo(jqId).get("device"));
        DeviceEntity deviceEntity = JSONObject.parseObject(deviceJson, DeviceEntity.class);
        if (deviceEntity == null) {
            throw new RRException("没有获取到设备信息,请联系管理员!");
        }
        //保存室内环境数据
        IndoorEnvironmentEntity indoorEnvironmentEntity = new IndoorEnvironmentEntity();
        Integer farmId = deviceEntity.getFarmId();
        indoorEnvironmentEntity.setFarmId(farmId);
        Integer pigstyId = deviceEntity.getPigstyId();
        indoorEnvironmentEntity.setPigstyId(pigstyId);
        indoorEnvironmentEntity.setValue(value);
        indoorEnvironmentEntity.setCollectTime(new Date());

        Map<String, Double> map = managementFeignService.selectByType((short) 1);
        switch (deviceDataEntity.getCmdType()) {
            case 7:
                indoorEnvironmentEntity.setType(1);
                handleDataPackageService.handleTemperature(indoorEnvironmentEntity, value, farmId, pigstyId, map);
                break;
            case 8:
                indoorEnvironmentEntity.setType(2);
                handleDataPackageService.handleHumidity(indoorEnvironmentEntity, value, farmId, pigstyId, map);
                break;
            case 9:
                indoorEnvironmentEntity.setType(3);
                handleDataPackageService.handleGas(indoorEnvironmentEntity, value, farmId, pigstyId, map);
                break;
            default:
                break;
        }
    }


    @Override
    public void interfaceTest(){
        Map<String, Double> map = managementFeignService.selectByType((short) 1);
        System.out.println(map);

    }


}
